--[[
Project name: HeadCount
Developed by: seppyk (http://www.authority-km.com)
Website: http://www.wowace.com/wiki/HeadCount
Description: Manages tracking of raid attendance and more.
License: Creative Common Attribution-NonCommercial-ShareAlike 3.0 Unported
File: Exporter.lua
File description: Raid exporting
]]

local L = AceLibrary("AceLocale-2.2"):new("HeadCount")

-- Exports a given raid in the configured format
-- @param raid The raid
-- @return string Returns the exported raid string
function HeadCount:exportRaid(raid)
	assert(raid, "Unable to export the raid because the raid is nil.")
	
	local exportString = nil
	
	local exportFormat = HeadCount:GetExportFormat()
	assert(exportFormat, "Unable to export raid because the export format is nil.")

	if ("EQdkp" == exportFormat) then
		-- EQdkp
		exportString = HeadCount:exportEQdkp(raid) 
	else
		-- phpBB
		exportString = HeadCount:exportPhpBB(raid) 
	end
	
	return exportString
end

-- Export a given raid in phpBB format
-- @param raid The raid
-- @return string Returns the exported phpBB string
function HeadCount:exportPhpBB(raid) 
	assert(raid, "Unable to export the raid into phpBB format because the raid is nil.")
	
	local exportString = ""	-- initialize as empty string
	
	local zone = raid:getZone()
	if (zone) then
		exportString = exportString .. "[b][u]" .. zone .. "[/b][/u]\n"   
	end 
		
    local raidStartTime = raid:retrieveStartingTime()
    local raidEndTime = raid:retrieveEndingTime()		
	exportString = exportString .. "[i]" .. HeadCount:getDateTimeAsString(raidStartTime) .. " - " .. HeadCount:getDateTimeAsString(raidEndTime) .."[/i]\n\n"
	
    local orderedPlayerList = raid:retrieveOrderedPlayerList("Name", true)	
    exportString = exportString .. "[b]" .. L["Players"] .. ":[/b]\n"
	if ((# orderedPlayerList) > 0) then 
		exportString = exportString .. "[list]\n"
		for k,v in ipairs(orderedPlayerList) do
			local waitTime = v:getWaitListTime()
			local offlineTime = v:getOfflineTime()		
			exportString = exportString .. "[*]" .. v:getName() .. " [i](" .. v:getClassName() .. ")[/i] | "
			exportString = exportString .. "[i]" .. L["Raid"] .. ": " .. HeadCount:getSecondsAsString(v:getRaidListTime()) .. ", [/i]"
			exportString = exportString .. "[i]" .. L["Standby"] .. ": " .. HeadCount:getSecondsAsString(v:getWaitListTime()) .. ", [/i]"
			exportString = exportString .. "[i]" .. L["Offline"] .. ": " .. HeadCount:getSecondsAsString(v:getOfflineTime()) .. "[/i]\n"
		end
		exportString = exportString .. "[/list]\n\n"
	else
		exportString = exportString .. L["None"] .. "\n\n"
	end

    if (raid:getBossList()) then
        -- boss list is present	
		exportString = exportString .. "[b]" .. L["Boss kills"] .. ":[/b]\n"
        local orderedBossList = raid:retrieveOrderedBossList()
		
		if ((# orderedBossList) > 0) then
			exportString = exportString .. "[list]\n"		
			for k,v in ipairs(orderedBossList) do
				exportString = exportString .. "[*]" .. v:getName() .. " at " .. HeadCount:getDateTimeAsString(v:getActivityTime()) .. "\n"
			end
			exportString = exportString .. "[/list]\n\n"
		else
			exportString = exportString .. L["None"] .. "\n\n"
		end
    end

	local lootList = raid:getLootList()
    exportString = exportString .. "[b]" .. L["Loot"] .. ":[/b]\n"
    
	if ((# lootList) > 0) then
		exportString = exportString .. "[list]\n"	
	    for k,v in ipairs(raid:getLootList()) do
			local lootName = v:getName()
			local color = HeadCount:retrieveItemColor(v:getRarity())
			if (lootName) then
				exportString = exportString .. "[*] [url=http://www.wowhead.com/?item=" .. v:getItemId() .. "][color=#" .. color .. "][b]" .. lootName .. "[/b][/color][/url] by " .. v:getPlayerName() .. "\n"
			else
				exportString = exportString .. "[*] [url=http://www.wowhead.com/?item=" .. v:getItemId() .. "][b]" .. L["Item unavailable"] .. "[/b][/url] by " .. v:getPlayerName() .. "\n"		
			end	
	    end
	    exportString = exportString .. "[/list]\n"	
	else
		exportString = exportString .. L["None"] .. "\n"	
	end
	
	return exportString
end

-- Export a given raid in EQdkp format
-- @param raid The raid
-- @return string Returns the exported EQdkp string
function HeadCount:exportEQdkp(raid) 
	assert(raid, "Unable to export the raid into EQdkp format because the raid is nil.")
	
	local raidStartTime = raid:retrieveStartingTime()	-- raid start time
	local raidEndTime = raid:retrieveEndingTime()		-- raid end time

	local exportString = "<RaidInfo>"	
	exportString = exportString .. "<key>" .. HeadCount:getEQdkpDateTimeAsString(raidStartTime) .. "</key>"
	exportString = exportString .. "<start>" .. HeadCount:getEQdkpDateTimeAsString(raidStartTime)  .. "</start>"
	exportString = exportString .. "<end>" .. HeadCount:getEQdkpDateTimeAsString(raidEndTime) .. "</end>"		
			
	local zone = raid:getZone()	-- raid zone
	if (zone) then 
		exportString = exportString .. "<zone>" .. zone .. "</zone>"	
	end
		
	local orderedPlayerList = raid:retrieveOrderedPlayerList("Name", true) 		
	exportString = exportString .. "<PlayerInfos>"
	for k,v in ipairs(orderedPlayerList) do
		exportString = exportString .. "<key" .. k .. ">"
		exportString = exportString .. "<name>" .. v:getName() .. "</name>"
		exportString = exportString .. "<race>" .. v:getRace() .. "</race>"
		if (v:getGuild()) then
			exportString = exportString .. "<guild>" .. v:getGuild() .. "</guild>"
		end
		exportString = exportString .. "<sex>" .. v:getSex() .. "</sex>"
		exportString = exportString .. "<class>" .. string.upper(v:getClassName()) .. "</class>"
		if (v:getLevel() > 0) then
			exportString = exportString .. "<level>" .. v:getLevel() .. "</level>"
		else
			exportString = exportString .. "<level>" .. HeadCount.DEFAULT_LEVEL .. "</level>"
		end
		exportString = exportString .. "</key" .. k .. ">"
	end
	exportString = exportString .. "</PlayerInfos>"

	-- <BossKills>
	-- 	<key1><name>%bossname%</name><time>%date%</time><attendees><key1><name>%playername1%</name><name>%playername2%</name></key1></attendees></key1>
	-- 	<key2><name>%bossname%</name><time>%date%</time><attendees><key1><name>%playername1%</name><name>%playername2%</name></key1></attendees></key2>	
	-- </BossKills>	
	exportString = exportString .. "<BossKills>"
	if (raid:getBossList()) then 
		-- boss list is present
		local orderedBossList = raid:retrieveOrderedBossList()
		for k,v in ipairs(orderedBossList) do 
			exportString = exportString .. "<key" .. k .. ">"
			exportString = exportString .. "<name>" .. v:getName() .. "</name>"
			exportString = exportString .. "<time>" .. HeadCount:getEQdkpDateTimeAsString(v:getActivityTime()) .. "</time>"
			exportString = exportString .. "<attendees>"
			
			local attendeeList = v:getPlayerList()
			for attendeeIndex,attendeeName in ipairs(attendeeList) do
				exportString = exportString .. "<key" .. attendeeIndex .. ">"
				exportString = exportString .. "<name>" .. attendeeName .. "</name>"
				exportString = exportString .. "</key" .. attendeeIndex .. ">"
			end
			
			exportString = exportString .. "</attendees>"
			exportString = exportString .. "</key" .. k .. ">"
		end
	end
	exportString = exportString .. "</BossKills>"		
		
	if (zone) then 
		exportString = exportString .. "<note><![CDATA[ - Zone: " .. zone .. "]]></note>"	
	else
		exportString = exportString .. "<note></note>"
	end
		
	exportString = exportString .. "<Join>"
	for k,v in ipairs(orderedPlayerList) do
		exportString = exportString .. "<key" .. k .. ">"
		exportString = exportString .. "<player>" .. v:getName() .. "</player>"
		exportString = exportString .. "<race>" .. v:getRace() .. "</race>"
		exportString = exportString .. "<class>" .. string.upper(v:getClassName()) .. "</class>"
		exportString = exportString .. "<sex>" .. v:getSex() .. "</sex>"			
		if (v:getLevel() > 0) then
			exportString = exportString .. "<level>" .. v:getLevel() .. "</level>"
		else
			exportString = exportString .. "<level>" .. HeadCount.DEFAULT_LEVEL .. "</level>"
		end
		exportString = exportString .. "<time>" ..HeadCount:getEQdkpDateTimeAsString(v:retrieveStartingTime()) .. "</time>"			
		exportString = exportString .. "</key" .. k .. ">"
	end		
	exportString = exportString .. "</Join>"

	exportString = exportString .. "<Leave>"
	for k,v in ipairs(orderedPlayerList) do
		exportString = exportString .. "<key" .. k .. ">"
		exportString = exportString .. "<player>" .. v:getName() .. "</player>"			
		exportString = exportString .. "<time>" .. HeadCount:getEQdkpDateTimeAsString(v:retrieveEndingTime()) .. "</time>"
		exportString = exportString .. "</key" .. k .. ">"
	end		
	exportString = exportString .. "</Leave>"
		
	exportString = exportString .. "<Loot>"
	for k,v in ipairs(raid:getLootList()) do
		exportString = exportString .. "<key" .. k .. ">"
		
		local lootName = v:getName()
		if (lootName) then
			exportString = exportString .. "<ItemName>" .. lootName .. "</ItemName>"
		end
		
		local lootId = v:getItemId() 
		if (lootId) then
			exportString = exportString .. "<ItemID>" .. lootId .. "</ItemID>" 
		end
		
		local textureIcon = v:retrieveTextureIcon()
		if (textureIcon) then 
			exportString = exportString .. "<Icon>" .. textureIcon .. "</Icon>" 
		end
		
		local lootItemType = v:getItemType()
		if (lootItemType) then
			exportString = exportString .. "<Class>" .. lootItemType .. "</Class>"
		end
		
		local lootItemSubType = v:getItemSubType()
		if (lootItemSubType) then
			exportString = exportString .. "<SubClass>" .. lootItemSubType .. "</SubClass>"
		end
		
		local lootColor = v:retrieveColor()
		if (lootColor) then
			exportString = exportString .. "<Color>" .. lootColor .. "</Color>"
		end
		exportString = exportString .. "<Count>" .. v:getQuantity() .. "</Count>"
		exportString = exportString .. "<Player>" .. v:getPlayerName() .. "</Player>"
		
		local lootCost = v:getCost()
		if (not lootCost) then
			lootCost = 0
		end				
		exportString = exportString .. "<Costs>" .. lootCost .. "</Costs>"
		exportString = exportString .. "<Time>" .. HeadCount:getEQdkpDateTimeAsString(v:getActivityTime()) .. "</Time>"
		
		local lootNote = "<![CDATA["		
		local lootZone = v:getZone()
		if (lootZone) then 
			-- loot zone present
			lootNote = lootNote .. " - Zone: " .. lootZone	-- add to note
			exportString = exportString .. "<Zone>" .. lootZone .. "</Zone>"	
		else
			exportString = exportString .. "<Zone />"
		end			
		
		local lootSource = v:getSource()
		if (lootSource) then
			-- loot source present
			lootNote = lootNote .. " - Boss: " .. lootSource -- add to note
			exportString = exportString .. "<Boss>" .. lootSource .. "</Boss>"
		else
			exportString = exportString .. "<Boss />"
		end
		
		lootNote = lootNote .. " - " .. lootCost .. " DKP]]>"	-- add cost to note
		exportString = exportString .. "<Note>" .. lootNote .. "</Note>"
		exportString = exportString .. "</key" .. k .. ">"			
	end		
	exportString = exportString .. "</Loot>"
		
	exportString = exportString .. "</RaidInfo>"
	
	return exportString	
end
